#define ENCODER_DO_NOT_USE_INTERRUPTS
#include "Encoder.h"
#include "MegunoLink.h"
#include "Filter.h"

//pin configuration
const int SDIO = 3;
const int SCLK = 2;
const int lButton = 6;
const int rButton = 7;
const int mButton = 8;
const int bWheelAdd = 9; //might not be needed
const int bWheelSub = 10; //might not be needed

// Create a new exponential filter with a weight of 40 and initial value of 0.
ExponentialFilter<long> ADCFilter(40, 0);

//Setup encoder object
Encoder myEnc(10, 9);

//setup initial synth mode
int programState = 2;
boolean filterOn = true;

//Current button state variables
// left, middle, right
boolean buttonStates[] = {0, 0, 0};
boolean prevButtonStates[] = {0, 0, 0};
boolean toggleState = false; //is the toggle button pushed
long position[2]  = {0, 0}; //encoder wheel positions

//debounce variables
long lastDebounceTime[] = {0, 0, 0, 0, 0};
long lastToggleTime = 0;
long lastStateTime = 0;
long lastFilterTime = 0;
long debounceDelay = 75;

//mouse sensor variables
int deltaX;
int deltaY;
int sensAverage;

//Sound variables
long highLimit = 4000;
long lowLimit = 31; //lowest frequency that works reliable on a Nano
long pitchCtr = 1250;
long rangeBase = 2500; //starting point to base ranges
long pitchRange = 2500;
long lowRange = pitchCtr - pitchRange / 2;
long highRange = pitchCtr + pitchRange / 2;
float pitchGranularity = 800; //number of divisions
float incrementCoef = float(pitchRange) / float(pitchGranularity);
float pitchVal = 500;
float adjVal = 500;
long changeAmt = 0; //pitch change when in camera mode with x adjust

boolean tremOn = false;
float tremRate = 0;
long toneOnTime = 0;
long toneOffTime = 0;
boolean setTime = true;
boolean toneOn = true;

void setup()
{
  pinMode(SCLK, OUTPUT);
  pinMode(SDIO, OUTPUT);
  pinMode(lButton, INPUT_PULLUP);
  pinMode(rButton, INPUT_PULLUP);
  pinMode(mButton, INPUT_PULLUP);
  pinMode(bWheelAdd, INPUT_PULLUP); //might not be needed
  pinMode(bWheelSub, INPUT_PULLUP); //might not be needed
  Serial.begin(38400);
  mouseInit();
  dumpDiag();
}

void loop()
{

  //read all pins
  deBounce(0, digitalRead(lButton));
  deBounce(1, digitalRead(mButton));
  deBounce(2, digitalRead(rButton));

  //read all variables from mouse
  sensAverage = readAverage();
  deltaX = readDeltaX();
  deltaY = readDeltaY();

  //if the opposite of the current mode button (momentary if latched, latched if momentary, move the center point.  Else change range)
  if ((buttonStates[0] == LOW && !toggleState) || (buttonStates[2] == LOW && buttonStates[0] == LOW && toggleState)) {

    long newPos = myEnc.read();

    if (newPos != position[0]) {//set pitchctr based on button plus encoder

      position[1] += (newPos - position[0]); //determines change in position 1 by amount of change in position 0
      myEnc.write(position[0]);

      if (position[1] < -25) {
        position[1] = -25;
      }
      if (position[1] > 20) {
        position[1] = 20;
      }
      pitchCtr = pitchCtr + (position[1] * 50);
      if (pitchCtr < lowLimit) {
        pitchCtr = lowLimit;
      }
      if (pitchCtr > highLimit) {
        pitchCtr = highLimit;
      }
    }
  }

  //toggle tremOn if button if left and middle pressed at the same time
  if (buttonStates[0] == LOW && buttonStates[1] == LOW) {
    tremOn = !tremOn;
  }

  else {  //set the range based on the encoder wheel
    long newPos = myEnc.read();
    if (newPos != position[0]) {
      position[0] = newPos;
      if (position[0] < -25) {
        myEnc.write(-25);
        position[0] = -25;
      }
      if (position[0] > 20) {
        myEnc.write(20);
        position[0] = 20;
      }
      pitchRange = rangeBase + position[0] * 100;
    }
  }

  lowRange = pitchCtr - pitchRange / 2;
  if (lowRange < lowLimit)lowRange = lowLimit;
  highRange = pitchCtr + pitchRange / 2;
  if (highRange > highLimit)highRange = highLimit;

  incrementCoef = float(pitchRange) / float(pitchGranularity);

  //Serial.println(String(buttonStates[0])+ String(buttonStates[1])+ String(buttonStates[2])+ String(toggleState));

  if (buttonStates[1] == LOW && buttonStates[0] == HIGH) {
    if (millis() > lastStateTime + 500) {
      programState++;
      if (programState > 2)programState = 0;
      lastStateTime = millis();
    }
  }

  if (buttonStates[2] == LOW) {
    if (millis() > lastToggleTime + 500) {
      toggleState = !toggleState;
      lastToggleTime = millis();
    }
  }

  switch (programState) {

    //determine pitch based on mode
    case 0: //oscillator from Xdelta
      pitchVal += (float(deltaX) * incrementCoef);
      break;

    case 1: //Oscillator from Camera
      pitchVal = float(pitchRange) / 114 * (float(sensAverage) - 17) + float(lowRange);
      break;

    case 2: //Oscillator from Xdelta and Camera (change to average two sources)
      pitchVal = float(pitchRange) / 114 * (float(sensAverage) - 17) + float(lowRange);
      adjVal += (float(deltaX) * incrementCoef);

      pitchVal = (float(pitchVal) * 2 + float(adjVal)) / 3;

      break;
  }

  //set tremolo rate from deltaY
  if (tremOn) {
    tremRate += (float(deltaY) * 50);
    if (tremRate < 3150)tremRate = 3150;
    if (tremRate > 88500 )tremRate = 88500;
  }
  else {
    tremRate = 0;
  }

  //limit pitch to range
  if (pitchVal < lowRange) {
    pitchVal = lowRange;
  }
  if (pitchVal > highRange) {
    pitchVal = highRange;
  }

  //filter data if filter is on
  if (filterOn) {
    ADCFilter.Filter(pitchVal);
    pitchVal = ADCFilter.Current();
  }

  //if momentary button is pressed make a sound
  if (toggleState) {
    if (buttonStates[0] == LOW) {

      if (tremRate > 2999) {
        if (toneOn) {
          if (micros() < toneOffTime) {
            tone(5, pitchVal);
          }
          else {
            toneOnTime = micros() + tremRate;
            toneOn = !toneOn;
          }
        }
        else {
          if (micros() < toneOnTime) {
            noTone(5);
          }
          else {
            toneOffTime = micros() + tremRate;
            toneOn = !toneOn;
          }
        }
      }
      else {
        toneOn = true;
        tone(5, pitchVal);
      }

    }
    else {
      noTone(5);
    }
  }

  else {
    if (tremRate > 2999) {
      if (toneOn) {
        if (micros() < toneOffTime) {
          tone(5, pitchVal);
        }
        else {
          toneOnTime = micros() + tremRate;
          toneOn = !toneOn;
        }
      }
      else {
        if (micros() < toneOnTime) {
          noTone(5);
        }
        else {
          toneOffTime = micros() + tremRate;
          toneOn = !toneOn;
        }
      }
    }
    else {
      toneOn = true;
      tone(5, pitchVal);
    }

  }

}

void deBounce(int buttonNo, int reading) {

  if (reading != prevButtonStates[buttonNo]) {
    lastDebounceTime[buttonNo] = millis();
  }

  if ((millis() - lastDebounceTime[buttonNo]) > debounceDelay) {

    // if the button state has changed:
    if (reading != buttonStates[buttonNo]) {
      buttonStates[buttonNo] = reading;

    }
  }
  prevButtonStates[buttonNo] = reading;
}
